// 一个模块（子目录）下的入口文件，统一命名为mod.rs，模块下的其他文件需要公开的，在mod.rs中统一导出
// 或者模块仅有单文件，则可以不用子目录而直接用单个文件作为一个模块，如random_test.rs

// [dependencies]
// rand = "0.7.3"

use rand::prelude::*;

pub fn test() {
    let rd = Random { seed: 100 };
    println!("a random number:{}", rd.next());
}

// 声明一个结构体
pub struct Random {
    pub seed: i32 // 成员变量
}

impl Random { // 实现结构体，添加成员函数
    pub fn next(&self) -> i32 { // lambda式语法
        // rust没有gc，变量离开其作用域时，立即释放，这是使rust能作为一个实时语言支持更严苛应用场景下的开发的最大魅力所在
        // 分配在栈上的基本类型、全由基本类型组成的元组变量或切片，赋值（包括作为参数或返回值传递）给另一个变量时都是深拷贝
        // 分配在堆上的字符串等复杂类型变量，赋值（包括作为参数或返回值传递）给另一个变量时不是浅拷贝（rust没有这个东西），而是直接移动，原变量立即失效
        // 如果希望原变量依然有效，不被移出，可以采用&引用符号传递变量引用，引用是更多一层指向原变量的指针，不是浅拷贝，这种方式称之为"借用"，且借来的引用只读不可改写，称之为"不可变借用"
        // 如果希望借来的引用可改写，则加上mut关键字即可，函数参数传递时型参和实参都要加，加mut的引用方式称之为"可变借用"；且有约束，同一个变量仅可有一个可变借用，多个可变借用，或可变和不可变借用并存都不行，但允许有多个不可变借用
        // 不能返回引用，因为离开作用域引用所指向的变量已经释放了，外部再使用其引用自然会报错；直接返回原变量，或者通过传入可变借用参数来解决
        let mut rng = rand::thread_rng();
        let mut nums : Vec<i32> = (1..*(&self.seed)).collect(); // 构造一个从1到seed的向量类型，向量类似于java的List，或go的切片，而相较于rust的数组它是不定长的，相较于rust的切片它是可变的
        nums.shuffle(&mut rng);
        nums[0] // 函数最后一行即返回值，所以可以用if else if实现三目表达式，最后一行表达式返回值类型需要与函数声明的返回类型一致，也有return用于提前返回；可用元组返回多值；无返回值的函数直接不要声明箭头和后面的返回值类型，或者用"空元组" -> ()
    }
}