use std::{thread, time};
use std::sync::RwLock;
use chrono::prelude::*;
use super::IProgramKeygen;

// 时间单位，一纳秒的多少倍，1e6 = 一毫秒，1e7 = 百分之一秒，1e8 = 十分之一秒
const SNOWFLAKE_TIME_UNIT: i64 = (1e7 as i64);
// 序列号的个数最多256个(0-255)，即每单位时间并发数，如时间单位是1e7，则单实例qps = 25600
const BIT_LEN_SEQUENCE: u16 = 8;
// 数据中心个数最多8个(0-7)，即同一个环境（生产、预发布、测试等）的数据中心（假设一个机房相同数据域的应用服务器集群只有一个，则数据中心数等于机房数）最多有8个
const BIT_LEN_DATA_CENTER_ID: u16 = 3;
// 同一个数据中心下最多65536个应用实例（0-65535），默认是根据实例ip后两段算实例id（k8s环境动态创建Pod，也建议用此方式），所以需要预留255 * 255这么多
const BIT_LEN_MACHINE_ID: u16 = 16;
// 时间戳之差最大 = 2的36次方 * 时间单位 / 1e9 秒，目前的设计最多可以用21.79年就需要更新开始时间（随之还需要归档旧数据和更新次新数据id）
const BIT_LEN_TIME: i64 = 1 << 36;
// 总共63位，不超过bit64

pub struct SnowflakeKeygen {
    start_time: i64,
    elapsed_time: RwLock<i64>,
    sequence: RwLock<u16>,
    data_center_id: u16,
    machine_id: u16,
}

fn to_snowflake_time(t: DateTime<Utc>) -> i64 {
    t.timestamp_nanos() / SNOWFLAKE_TIME_UNIT
}

fn get_current_elapsed_time(start_time: i64) -> i64 {
    to_snowflake_time(Utc::now()) - start_time
}

fn get_sleep_time(overtime: i64) -> time::Duration {
    time::Duration::from_nanos(overtime as u64) * 10000000 -
        time::Duration::from_nanos((Utc::now().timestamp_nanos() % SNOWFLAKE_TIME_UNIT) as u64)
}

impl IProgramKeygen for SnowflakeKeygen {
    fn next_id(&self) -> u64 {
        const MASK_SEQUENCE: u16 = 1 << BIT_LEN_SEQUENCE - 1;

        let mut elapsed_time = self.elapsed_time.write().unwrap();
        let mut sequence = self.sequence.write().unwrap();

        let current = get_current_elapsed_time(self.start_time);
        if *elapsed_time < current {
            *elapsed_time = current;
            *sequence = 0;
        } else { // self.elapsed_time.get() >= current
            *sequence = (*sequence + 1) & MASK_SEQUENCE;
            if *sequence == 0 {
                *elapsed_time += 1;
                let overtime = *elapsed_time - current;
                thread::sleep(get_sleep_time(overtime));
            }
        }

        self.to_id(*elapsed_time, *sequence)
    }
}

impl SnowflakeKeygen {
    pub fn new() -> SnowflakeKeygen {
        SnowflakeKeygen {
            start_time: to_snowflake_time(Utc.ymd(2019, 7, 1).and_hms_nano(0, 0, 0, 0_000_000)),
            elapsed_time: RwLock::new(0),
            sequence: RwLock::new(1 << BIT_LEN_SEQUENCE - 1),
            data_center_id: 0,
            machine_id: 0,
        }
    }

    fn to_id(&self, elapsed_time: i64, sequence: u16) -> u64 {
        if elapsed_time >= BIT_LEN_TIME {
            return 0;
        }

        (elapsed_time as u64) << ((BIT_LEN_SEQUENCE + BIT_LEN_DATA_CENTER_ID + BIT_LEN_MACHINE_ID) as u64) |
            (sequence as u64) << ((BIT_LEN_DATA_CENTER_ID + BIT_LEN_MACHINE_ID) as u64) |
            (self.data_center_id as u64) << (BIT_LEN_MACHINE_ID as u64) |
            (self.machine_id as u64)
    }
}